<?php

namespace Drupal\static_build\Plugin;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Component\Plugin\Discovery\CachedDiscoveryInterface;
use Drupal\Core\Cache\CacheableDependencyInterface;

/**
 * Typed interface for Static Builder Manager.
 */
interface StaticBuilderManagerInterface extends PluginManagerInterface, CachedDiscoveryInterface, CacheableDependencyInterface {

  /**
   * {@inheritdoc}
   *
   * Wraps parent method to add typing.
   *
   * @return \Drupal\static_build\Plugin\StaticBuilderInterface
   *   A exporter object instance.
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function createInstance($plugin_id, array $configuration = []): StaticBuilderInterface;

  /**
   * {@inheritdoc}
   *
   * Wraps parent method to add typing.
   *
   * @return \Drupal\static_build\Plugin\StaticBuilderInterface
   *   A exporter object instance.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   *   If the instance cannot be created, such as if the ID is invalid.
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function getInstance(array $options): StaticBuilderInterface;

  /**
   * Get local definitions.
   *
   * @return array
   *   Local definitions
   */
  public function getLocalDefinitions(): array;

  /**
   * Get cloud definitions.
   *
   * @return array
   *   Cloud definitions
   */
  public function getCloudDefinitions(): array;

  /**
   * Filter definitions by host name.
   *
   * @param string $mode
   *   Local or cloud.
   *
   * @return array
   *   Filtered definitions
   */
  public function getDefinitionsByHostMode(string $mode): array;

}
