<?php

namespace Drupal\static_build\Routing;

use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\static_build\Plugin\StaticBuilderInterface;
use Drupal\static_build\Plugin\StaticBuilderManagerInterface;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Listens to the dynamic route events.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * Static Builder Manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderManagerInterface
   */
  protected $staticBuilderManager;

  /**
   * Constructs the subscriber.
   *
   * @param \Drupal\static_build\Plugin\StaticBuilderManagerInterface $staticBuilderManager
   *   Static Builder Manager.
   */
  public function __construct(StaticBuilderManagerInterface $staticBuilderManager) {
    $this->staticBuilderManager = $staticBuilderManager;
  }

  /**
   * {@inheritdoc}
   */
  public function alterRoutes(RouteCollection $collection) {
    // Override NodeViewController.
    if ($route = $collection->get('entity.node.canonical')) {
      $routeDefaults = $route->getDefaults();
      $routeDefaults['_controller'] = '\Drupal\static_build\Controller\NodeViewController::view';
      $route->setDefaults($routeDefaults);
    }

    // Create routes for builders.
    $runModes = [StaticBuilderInterface::RUN_MODE_LIVE, StaticBuilderInterface::RUN_MODE_PREVIEW];
    foreach ($runModes as $runMode) {
      if ($runMode === StaticBuilderInterface::RUN_MODE_LIVE) {
        $builderDefinitions = $this->staticBuilderManager->getDefinitions();
      }
      else {
        $builderDefinitions = $this->staticBuilderManager->getLocalDefinitions();
      }
      if (is_array($builderDefinitions) && count($builderDefinitions) > 0) {
        // Add a default route for the first builder.
        $route = new Route(
          '/admin/reports/static/build/' . $runMode . '/releases/list',
          [
            '_controller' => '\Drupal\static_build\Controller\ReleaseController::listReleases',
            '_title' => 'Static Build - ' . ucfirst($runMode) . ' releases',
            'runMode' => $runMode,
            'builderId' => array_keys($builderDefinitions)[0],
          ],
          ['_permission' => 'administer site configuration'],
          ['_admin_route' => TRUE]
        );
        $collection->add('static_build.releases_list.' . $runMode . '.default', $route);

        // Add another route for all builders.
        $route = new Route(
            '/admin/reports/static/build/' . $runMode . '/releases/list/{builderId}',
            [
              '_controller' => '\Drupal\static_build\Controller\ReleaseController::listReleases',
              '_title' => 'Static Build - ' . ucfirst($runMode) . ' releases',
              'runMode' => $runMode,
            ],
            ['_permission' => 'administer site configuration'],
            ['_admin_route' => TRUE]
          );
        $collection->add('static_build.releases_list.' . $runMode, $route);
      }
    }
  }

}
