<?php

namespace Drupal\static_deploy\Plugin;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\static_suite\Plugin\CacheablePluginManager;
use Traversable;

/**
 * Provides the Static Deployer plugin manager.
 */
class StaticDeployerManager extends CacheablePluginManager implements StaticDeployerManagerInterface {

  /**
   * Constructs a new StaticDeployerManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/StaticDeployer', $namespaces, $module_handler, 'Drupal\static_deploy\Plugin\StaticDeployerInterface', 'Drupal\static_deploy\Annotation\StaticDeployer');

    $this->alterInfo('static_deploy_static_deployer_info');
    $this->setCacheBackend($cache_backend, 'static_deploy_static_deployer_plugins');
  }

  /**
   * {@inheritdoc}
   *
   * Wraps original createInstance() to add typing.
   *
   * @return StaticDeployerInterface
   *   A static deployer object instance.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   *   If the instance cannot be created, such as if the ID is invalid.
   */
  public function createInstance($plugin_id, array $configuration = []): StaticDeployerInterface {
    $instance = parent::createInstance($plugin_id, $configuration);
    if ($instance instanceof StaticDeployerInterface) {
      return $instance;
    }

    throw new PluginException('Plugin is not an instance of StaticDeployerInterface');
  }

  /**
   * {@inheritdoc}
   *
   * Wraps parent method to add typing.
   *
   * @return StaticDeployerInterface
   *   A static builder object instance.
   */
  public function getInstance(array $options): StaticDeployerInterface {
    $instance = parent::getInstance($options);
    if ($instance instanceof StaticDeployerInterface) {
      return $instance;
    }

    throw new PluginException('Plugin is not an instance of StaticDeployerInterface');
  }

}
