<?php

namespace Drupal\static_deploy\Routing;

use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\static_build\Plugin\StaticBuilderManagerInterface;
use Drupal\static_deploy\Plugin\StaticDeployerManagerInterface;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Listens to the dynamic route events.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * Static Builder Manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderManagerInterface
   */
  protected $staticBuilderManager;

  /**
   * Static Deployer Manager.
   *
   * @var \Drupal\static_deploy\Plugin\StaticDeployerManagerInterface
   */
  protected $staticDeployerManager;

  /**
   * Constructs the subscriber.
   *
   * @param \Drupal\static_build\Plugin\StaticBuilderManagerInterface $staticBuilderManager
   *   Static Builder Manager.
   * @param \Drupal\static_deploy\Plugin\StaticDeployerManagerInterface $staticDeployerManager
   *   Static Deployer Manager.
   */
  public function __construct(StaticBuilderManagerInterface $staticBuilderManager, StaticDeployerManagerInterface $staticDeployerManager) {
    $this->staticBuilderManager = $staticBuilderManager;
    $this->staticDeployerManager = $staticDeployerManager;
  }

  /**
   * {@inheritdoc}
   */
  public function alterRoutes(RouteCollection $collection) {
    $localBuilderDefinitions = $this->staticBuilderManager->getLocalDefinitions();
    $deployerDefinitions = $this->staticDeployerManager->getDefinitions();
    if (is_array($localBuilderDefinitions) && count($localBuilderDefinitions) > 0 && is_array($deployerDefinitions) && count($deployerDefinitions) > 0) {
      foreach ($deployerDefinitions as $deployerDefinition) {
        // Add a default route for the first builder.
        $route = new Route(
        '/admin/reports/static/deploy/' . $deployerDefinition['id'] . '/live/releases/list',
        [
          '_controller' => '\Drupal\static_deploy\Controller\ReleaseController::listReleases',
          '_title' => 'Static Deploy - Releases deployed to ' . $deployerDefinition['label'],
          'deployerId' => $deployerDefinition['id'],
          'builderId' => array_keys($localBuilderDefinitions)[0],
        ],
        ['_permission' => 'administer site configuration'],
        ['_admin_route' => TRUE]
        );
        $collection->add('static_deployer_' . $deployerDefinition['id'] . '.releases_list.live.default', $route);

        // Add another route for all builders.
        $route = new Route(
            '/admin/reports/static/deploy/' . $deployerDefinition['id'] . '/live/releases/list/{builderId}',
            [
              '_controller' => '\Drupal\static_deploy\Controller\ReleaseController::listReleases',
              '_title' => 'Static Deploy - Releases deployed to ' . $deployerDefinition['label'],
              'deployerId' => $deployerDefinition['id'],
            ],
            ['_permission' => 'administer site configuration'],
             ['_admin_route' => TRUE]
          );
        $collection->add('static_deployer_' . $deployerDefinition['id'] . '.releases_list.live', $route);
      }
    }
  }

}
