<?php

namespace Drupal\static_export_graphql_resolver\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\static_export\Entity\ExportableEntityInterface;
use Drupal\static_export\Event\ExportableEntityListEvent;
use Drupal\static_export\Event\ExportableEntityListEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for Exportable Entity List.
 */
class ExportableEntityListEventSubscriber implements EventSubscriberInterface {
  use StringTranslationTrait;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ExportableEntityListEvents::ROW_BUILT][] = ['onRowBuilt'];
    return $events;
  }

  /**
   * Reacts to a ExportableEntityListEvents::ROW_BUILT event.
   *
   * @param \Drupal\static_export\Event\ExportableEntityListEvent $event
   *   The Exportable Entity List event.
   *
   * @return \Drupal\static_export\Event\ExportableEntityListEvent
   *   The processed event.
   */
  public function onRowBuilt(ExportableEntityListEvent $event): ExportableEntityListEvent {
    $eventData = $event->getData();
    if (!empty($eventData['entity']) && $eventData['entity'] instanceof ExportableEntityInterface && $eventData['entity']->getResolver() === 'graphql' && !empty($eventData['row'])) {
      $graphqlFile = $eventData['entity']->id() . '.gql';
      $gqlFileAbsolute = $this->configFactory->get('static_export_graphql_resolver.settings')
        ->get('dir') . '/' . $graphqlFile;
      $eventData['row']['resolver'] = [
        'data' =>
        [
          '#markup' => $eventData['row']['resolver'] . '<br/>(' .
          $this->t(
              'query defined in <abbr title="absolute path: @graphql_file_absolute">%graphql_file</abbr>',
              [
                '@graphql_file_absolute' => $gqlFileAbsolute,
                '%graphql_file' => $graphqlFile,
              ]
          )
          . ')',
        ],
      ];
      $event->setData($eventData);
    }
    return $event;
  }

}
