<?php

namespace Drupal\static_export_graphql_resolver;

use Drupal\graphql\GraphQL\Execution\QueryProcessor;
use Drupal\static_suite\Entity\EntityUtils;
use Drupal\static_suite\StaticSuiteUserException;
use GraphQL\Server\OperationParams;

/**
 * A handler for querying GraphQL.
 */
class GraphqlQueryHandler implements GraphqlQueryHandlerInterface {

  /**
   * GraphQL query processor.
   *
   * @var \Drupal\graphql\GraphQL\Execution\QueryProcessor
   */
  protected $queryProcessor;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtils
   */
  protected $entityUtils;

  /**
   * Constructor.
   *
   * @param \Drupal\graphql\GraphQL\Execution\QueryProcessor $queryProcessor
   *   GraphQL query processor.
   * @param \Drupal\static_suite\Entity\EntityUtils $entityUtils
   *   Entity utils service.
   */
  public function __construct(QueryProcessor $queryProcessor, EntityUtils $entityUtils) {
    $this->queryProcessor = $queryProcessor;
    $this->entityUtils = $entityUtils;
  }

  /**
   * {@inheritdoc}
   */
  public function query(string $graphqlQuery, array $variables = []): array {
    // Ensure "language" format (a common variable needed for querying multilingual
    // sites) is correct.
    if (isset($variables['language'])) {
      $variables['language'] = strtoupper(str_replace('-', '_', $variables['language']));
    }

    $bodyParams = [
      'query' => $graphqlQuery,
      'variables' => json_encode($variables),
      'operationName' => NULL,
    ];

    $params = OperationParams::create($bodyParams, TRUE);
    $result = $this->queryProcessor->processQuery('default:default', $params);

    if (!empty($result->errors) && is_array($result->errors)) {
      throw new StaticSuiteUserException('Error on query: ' . $result->errors[0]);
    }

    return $result->toArray();
  }

}
