<?php

namespace Drupal\static_export_graphql_resolver\Plugin\GraphQL\Fields\Entity;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql\Plugin\GraphQL\Fields\FieldPluginBase;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use GraphQL\Type\Definition\ResolveInfo;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @GraphQLField(
 *   id = "config_include",
 *   secure = false,
 *   name = "configInclude",
 *   description = "Path to the config file to be included",
 *   type = "String",
 *   parents= {"Entity"},
 *   arguments = {
 *     "name" = "String!"
 *   },
 *   nullable = true
 * )
 */
class ConfigInclude extends FieldPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The exporter manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Constructs a Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ExporterManagerInterface $exporterManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configuration = $configuration;
    $this->pluginId = $plugin_id;
    $this->pluginDefinition = $plugin_definition;
    $this->exporterManager = $exporterManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.static_exporter')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    if ($value instanceof EntityInterface) {
      $configExporter = $this->exporterManager->getInstance(['plugin_id' => 'config']);
      $configExporter->setOptions(['config-name' => $args['name']]);
      yield $configExporter->getRelativeFilePath();
    }
  }

}
