<?php

namespace Drupal\static_export_graphql_resolver\Plugin\GraphQL\Fields\Entity;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql\Plugin\GraphQL\Fields\FieldPluginBase;
use Drupal\static_export\Plugin\CustomExporterInterface;
use GraphQL\Type\Definition\ResolveInfo;
use InvalidArgumentException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @GraphQLField(
 *   id = "custom_include",
 *   secure = false,
 *   name = "customInclude",
 *   description = "Path to the custom file to be included",
 *   type = "String",
 *   parents= {"Entity"},
 *   arguments = {
 *     "path" = "String!"
 *   },
 *   nullable = true
 * )
 */
class CustomInclude extends FieldPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The exporter manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configuration = $configuration;
    $this->pluginId = $plugin_id;
    $this->pluginDefinition = $plugin_definition;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    if ($value instanceof EntityInterface) {
      // Avoid path traversals.
      $fullPath = realpath($this->configFactory->get('static_export.settings')->get('data_dir') . CustomExporterInterface::BASE_DIR . $args['path']);
      if (strpos($fullPath, $this->configFactory->get('static_export.settings')->get('data_dir') . CustomExporterInterface::BASE_DIR) !== 0) {
        throw new InvalidArgumentException('"' . $args['path'] . '" is not a valid path');
      }
      yield CustomExporterInterface::BASE_DIR . $args['path'];
    }
  }

}
