<?php

namespace Drupal\static_export_json_api_resolver\Plugin\Resolver;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\static_export\Plugin\ResolverInterface;
use Drupal\static_export_json_api_resolver\JsonApiRequestHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A JSON:API resolver.
 *
 * @StaticResolver(
 *  id = "json_api",
 *  label = "JSON:API",
 *  description = "JSON:API data resolver",
 *  format = "JSON"
 * )
 */
class JsonApiResolver extends PluginBase implements ResolverInterface, ContainerFactoryPluginInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Handler for requesting data from JSON:API.
   *
   * @var \Drupal\static_export_json_api_resolver\JsonApiRequestHandlerInterface
   */
  protected $jsonApiRequestHandler;

  /**
   * Constructor for exporter.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\static_export_json_api_resolver\JsonApiRequestHandlerInterface $jsonApiRequestHandler
   *   Handler for requesting data from JSON:API.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    JsonApiRequestHandlerInterface $jsonApiRequestHandler
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->jsonApiRequestHandler = $jsonApiRequestHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('static_export_json_api_resolver.json_api_request_handler')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  public function resolve(EntityInterface $entity): string {
    $path = $this->configFactory->get('static_export_json_api_resolver.settings')->get('endpoint') . '/' . $entity->getEntityTypeId() . '/' . $entity->bundle() . '/' . $entity->uuid();
    $queryParams = [];
    $configQueryParams = $this->configFactory->get('static_export_json_api_resolver.settings')->get('query_params.' . $entity->getEntityTypeId() . '.' . $entity->bundle());
    if ($configQueryParams) {
      $queryParams = explode('&', $configQueryParams);
    }
    return $this->jsonApiRequestHandler->request($path, $queryParams);
  }

  /**
   * {@inheritdoc}
   */
  public function getVariantKeys(EntityInterface $entity): array {
    // No support for variants.
    return [];
  }

}
