<?php

namespace Drupal\static_export\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;

/**
 * Provides a listing of Exportable configurations.
 */
class ExportableConfigurationList extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public function listExportableConfigurations() {
    $config = $this->config('static_export.settings');

    $tableHeader = [
      'config_name' => $this->t('Configuration object name'),
      'export_file' => $this->t('Export file'),
      'modification_date' => $this->t('Last modification date'),
      'operations' => $this->t('Operations'),
    ];
    $tableRows = [];

    $configObjectsToExport = $config->get('exportable_config.objects_to_export');
    if (is_array($configObjectsToExport)) {
      foreach ($configObjectsToExport as $configName) {
        $exportFile = $config->get('data_dir') . $config->get('exportable_config.directory') . '/' . $configName . '.' . $config->get('exportable_config.format');
        $modificationDate = @filemtime($exportFile);
        $tableRows[] = [
          'config_name' => $configName,
          'export_file' => $config->get('data_dir') . $config->get('exportable_config.directory') . '/' . $configName . '.' . $config->get('exportable_config.format'),
          'modification_date' => $modificationDate ? date('Y-m-d H:i:s', $modificationDate) : $this->t('Not available'),
          'operations' => [
            'data' => [
              '#type' => 'operations',
              '#links' => [
                'view' => [
                  'title' => $this->t('View'),
                  'weight' => 10,
                  'url' => Url::fromRoute('static_export.file_viewer', [], ['query' => ['path' => $exportFile]]),
                ],
                'delete' => [
                  'title' => $this->t('Delete'),
                  'weight' => 10,
                  'url' => Url::fromRoute('static_export.exportable_config.delete_form', ['exportable_configuration' => $configName]),
                ],
              ],
            ],
          ],
        ];
      }
    }

    if (count($tableRows) === 0) {
      $tableRows[] = [
        'config_name' => $this->t('No exportable configuration available'),
        'export_file' => NULL,
        'operations' => [],
      ];
    }

    return [
      '#type' => 'table',
      '#header' => $tableHeader,
      '#rows' => $tableRows,
    ];

  }

}
