<?php

namespace Drupal\static_export\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a listing of Exportable translation.
 */
class ExportableTranslationList extends ControllerBase {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $languageManager;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(LanguageManagerInterface $language_manager) {
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function listExportableTranslations() {
    $config = $this->config('static_export.settings');

    $tableHeader = [
      'export_file' => $this->t('Export file'),
      'modification_date' => $this->t('Last export date'),
      'operations' => $this->t('Operations'),
    ];
    $tableRows = [];

    foreach ($this->languageManager->getLanguages() as $language) {
      $exportFile = $config->get('data_dir') . $config->get('exportable_translation.directory') . '/translation.' . $language->getId() . '.' . $config->get('exportable_translation.format');
      $modificationDate = @filemtime($exportFile);
      $tableRows[] = [
        'export_file' => $exportFile,
        'modification_date' => $modificationDate ? date('Y-m-d H:i:s', $modificationDate) : $this->t('Not available'),
        'operations' => [
          'data' => [
            '#type' => 'operations',
            '#links' => [
              'view' => [
                'title' => $this->t('View'),
                'weight' => 10,
                'url' => Url::fromRoute('static_export.file_viewer', [], ['query' => ['path' => $exportFile]]),
              ],
            ],
          ],
        ],
      ];
    }

    return [
      '#type' => 'table',
      '#header' => $tableHeader,
      '#rows' => $tableRows,
      '#prefix' => '<p>' . $this->t(
          'Each available language is exported to a file named <em>"translation.{LANG_CODE}"</em>. You can configure where is that file saved in the <a href="@settings_url">settings</a> page.',
        ['@settings_url' => Url::fromRoute('static_export.exportable_translation.settings')->toString()]
      ) . '</p>',
    ];

  }

}
