<?php

namespace Drupal\static_export\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\static_export\Service\NodePathResolverService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Defines a controller to resolver paths to exported files.
 */
class NodePathResolverController extends ControllerBase {

  /**
   * Node path resolver service.
   *
   * @var \Drupal\static_export\Service\NodePathResolverService
   */
  protected $nodePathResolverService;

  /**
   * PathResolver controller constructor.
   *
   * @param \Drupal\static_export\Service\NodePathResolverService $nodePathResolverService
   *   Drupal entity manager.
   */
  public function __construct(
    NodePathResolverService $nodePathResolverService
  ) {
    $this->nodePathResolverService = $nodePathResolverService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('static_export.node_path_resolver_service')
    );
  }

  /**
   * Given a node id, it returns its exported file path.
   *
   * @param string $nid
   *   Node id to search for.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response.
   */
  public function nodeIdToPathResolver(string $nid) {
    $relativePath = $this->nodePathResolverService->getExportedFilePathRelativeByNodeId($nid);
    return new JsonResponse(['relative-path' => $relativePath]);
  }

  /**
   * Given a node url, it returns its exported file path.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response.
   */
  public function nodeUrlToPathResolver(Request $request) {
    $nodeUrl = $request->query->get('node-url');
    $relativePath = NULL;
    if ($nodeUrl) {
      $relativePath = $this->nodePathResolverService->getExportedFilePathRelativeByNodeUrl($nodeUrl);
    }
    return new JsonResponse(['relative-path' => $relativePath]);
  }

}
