<?php

namespace Drupal\static_export\Entity;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityInterface;
use Drupal\static_suite\Entity\EntityUtilsInterface;

/**
 *
 */
class ExportableEntityManager implements ExportableEntityManagerInterface {

  /**
   * Entity utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtilsInterface
   */
  protected $entityUtils;

  /**
   * ExportableEntityManager constructor.
   *
   * @param \Drupal\static_suite\Entity\EntityUtilsInterface $entityUtils
   *   Entity utils.
   */
  public function __construct(EntityUtilsInterface $entityUtils) {
    $this->entityUtils = $entityUtils;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEnabledEntityTypes(): array {
    /* @var $exportableEntityIds \Drupal\static_export\Entity\ExportableEntityInterface[] */
    $exportableEntityIds = $this->entityUtils->getEntityIds('exportable_entity');
    $enabledEntityTypes = [];
    foreach ($exportableEntityIds as $exportableEntityId) {
      $exportableEntity = $this->entityUtils->loadEntity('exportable_entity', $exportableEntityId);
      if ($exportableEntity) {
        $enabledEntityTypes[] = $exportableEntity->getEntityType();
      }
    }
    return array_unique($enabledEntityTypes);
  }

  /**
   * {@inheritdoc}
   */
  public function getExportableEntity(EntityInterface $entity): ?ExportableEntityInterface {
    try {
      $exportableEntity = $this->entityUtils->loadEntity(
        'exportable_entity',
        $entity->getEntityTypeId() . '.' . $entity->bundle()
      );
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      return NULL;
    }

    if ($exportableEntity instanceof ExportableEntityInterface) {
      return $exportableEntity;
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isExportable(EntityInterface $entity): bool {
    $exportableEntity = $this->getExportableEntity($entity);
    return $exportableEntity && $exportableEntity->status();
  }

}
