<?php

namespace Drupal\static_export\Entity;

use Drupal\Core\Entity\EntityInterface;

/**
 * An interface for a manager of Exportable Entities.
 */
interface ExportableEntityManagerInterface {

  /**
   * Get which entity types are enabled to be exported.
   *
   * @return array
   *   Entity type.
   */
  public function getEnabledEntityTypes(): array;

  /**
   * Given an entity, get its ExportableEntity with its export config.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity.
   *
   * @return \Drupal\static_export\Entity\ExportableEntityInterface|null
   *   The ExportableEntity for the $entity param
   */
  public function getExportableEntity(EntityInterface $entity): ?ExportableEntityInterface;

  /**
   * Tells whether an entity is exportable (exists and it's enabled).
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity to be checked.
   *
   * @return bool
   *   True if exportable, false otherwise.
   */
  public function isExportable(EntityInterface $entity): bool;

}
