<?php

namespace Drupal\static_export\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form handler for adding exportable configurations.
 */
class ExportableConfigAddForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_export_exportable_config_add';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_export.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $currentConfigObjects = $config->get('exportable_config.objects_to_export');

    $allConfigNames = $this->configFactory->listAll();
    foreach ($allConfigNames as $configName) {
      if (!is_array($currentConfigObjects) || !in_array($configName, $currentConfigObjects, TRUE)) {
        $options[$configName] = $configName;
      }
    }

    $form['object_to_export'] = [
      '#type' => 'select',
      '#options' => $options,
      '#title' => $this->t('Select configuration object to export'),
      '#required' => TRUE,
    ];

    $form = parent::buildForm($form, $form_state);
    $form['actions']['submit']['#value'] = $this->t('Add');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $currentConfigObjects = $config->get('exportable_config.objects_to_export');

    if (is_array($currentConfigObjects) && in_array($form_state->getValue('object_to_export'), $currentConfigObjects, TRUE)) {
      $form_state->setErrorByName(
        'data_dir',
        $this->t('Configuration "@config" is already exported.', ['@config', $form_state->getValue('object_to_export')]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $configObjects = $config->get('exportable_config.objects_to_export');
    $configObjects[] = $form_state->getValue('object_to_export');
    sort($configObjects);
    $config
      ->set('exportable_config.objects_to_export', array_values($configObjects))
      ->save();

    $this->messenger()->addMessage($this->t('The configuration %label has been saved.', [
      '%label' => $form_state->getValue('object_to_export'),
    ]));

    $form_state->setRedirect('static_export.exportable_config.collection');
  }

}
