<?php

namespace Drupal\static_export\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Form handler for deleting exportable configurations.
 */
class ExportableConfigDeleteForm extends ConfirmFormBase {

  /**
   * Name of the configuration object to be deleted.
   *
   * @var string
   */
  protected $exportableConfiguration;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_export_exportable_config_delete';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete %name?', ['%name' => $this->exportableConfiguration]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('static_export.exportable_config.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $exportable_configuration = '') {
    $this->exportableConfiguration = $exportable_configuration;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->getEditable('static_export.settings');
    $configObjects = $config->get('exportable_config.objects_to_export');
    if (($key = array_search($this->exportableConfiguration, $configObjects)) !== FALSE) {
      unset($configObjects[$key]);
    }
    sort($configObjects);
    $config
      ->set('exportable_config.objects_to_export', array_values($configObjects))
      ->save();

    $this->messenger()->addMessage($this->t('The configuration %label has been deleted.', [
      '%label' => $this->exportableConfiguration,
    ]));

    $form_state->setRedirect('static_export.exportable_config.collection');
  }

}
