<?php

namespace Drupal\static_export\Plugin;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\static_suite\Plugin\CacheablePluginManager;
use Traversable;

/**
 * Provides the Exporter plugin manager.
 */
class ExporterManager extends CacheablePluginManager implements ExporterManagerInterface {

  /**
   * Constructs a new StaticExporter object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/Exporter', $namespaces, $module_handler, 'Drupal\static_export\Plugin\ExporterInterface', 'Drupal\static_export\Annotation\StaticExporter');

    $this->alterInfo('static_export_exporter_info');
    $this->setCacheBackend($cache_backend, 'static_export_exporter_plugins');
  }

  /**
   * {@inheritdoc}
   *
   * Wraps parent method to add typing.
   *
   * @return \Drupal\static_export\Plugin\ExporterInterface
   *   A exporter object instance.
   */
  public function createInstance($plugin_id, array $configuration = []): ExporterInterface {
    $instance = parent::createInstance($plugin_id, $configuration);
    if ($instance instanceof ExporterInterface) {
      return $instance;
    }

    throw new PluginException('Plugin is not an instance of ExporterInterface');
  }

  /**
   * {@inheritdoc}
   *
   * Wraps parent method to add typing.
   *
   * @return \Drupal\static_export\Plugin\ExporterInterface
   *   A exporter object instance.
   */
  public function getInstance(array $options): ExporterInterface {
    $instance = parent::getInstance($options);
    if ($instance instanceof ExporterInterface) {
      return $instance;
    }

    throw new PluginException('Plugin is not an instance of ExporterInterface');
  }

}
