<?php

namespace Drupal\static_export\Plugin;

/**
 * Defines the output of a exporter.
 */
class ExporterOutputConfig {

  /**
   * Export directory.
   *
   * @var string
   */
  protected $dir;

  /**
   * Export file name.
   *
   * @var string
   */
  protected $filename;

  /**
   * Export extension.
   *
   * @var string
   */
  protected $extension;

  /**
   * Export format.
   *
   * @var string
   */
  protected $format;

  /**
   * Constructor.
   *
   * @param string $dir
   *   Export directory.
   * @param string $filename
   *   Export file name.
   * @param string $extension
   *   Export extension.
   * @param string $format
   *   Export format.
   */
  public function __construct(string $dir, string $filename, string $extension, string $format) {
    $this->dir = $dir;
    $this->filename = $filename;
    $this->extension = $extension;
    $this->format = $format;
  }

  /**
   * Get export directory.
   *
   * @return string
   */
  public function getDir(): string {
    return $this->dir;
  }

  /**
   * Get export file name.
   *
   * @return string
   */
  public function getFilename(): string {
    return $this->filename;
  }

  /**
   * Get export extension.
   *
   * @return string
   */
  public function getExtension(): string {
    return $this->extension;
  }

  /**
   * Get export format.
   *
   * @return string
   */
  public function getFormat(): string {
    return $this->format;
  }

  /**
   * Format this object as a string.
   *
   * @return string
   */
  public function __toString(): string {
    return $this->getDir() . "/" . $this->getFilename() . "." . $this->getExtension();
  }

}
