<?php

namespace Drupal\static_export\Plugin;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\static_suite\Plugin\CacheablePluginManager;
use Traversable;

/**
 * Provides the Static Output Formatter plugin manager.
 */
class OutputFormatterManager extends CacheablePluginManager implements OutputFormatterManagerInterface {

  /**
   * Simple cache to store plugin objects.
   *
   * @var array
   */
  protected $cache;

  /**
   * Constructs a new OutputFormatterManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/OutputFormatter', $namespaces, $module_handler, 'Drupal\static_export\Plugin\OutputFormatterInterface', 'Drupal\static_export\Annotation\StaticOutputFormatter');

    $this->alterInfo('static_export_output_formatter_info');
    $this->setCacheBackend($cache_backend, 'static_export_output_formatter_plugins');
  }

  /**
   * {@inheritdoc}
   *
   * Wraps parent method to add typing.
   *
   * @return \Drupal\static_export\Plugin\OutputFormatterInterface
   *   A formatter object instance.
   */
  public function createInstance($plugin_id, array $configuration = []): OutputFormatterInterface {
    $instance = parent::createInstance($plugin_id, $configuration);
    if ($instance instanceof OutputFormatterInterface) {
      return $instance;
    }

    throw new PluginException('Plugin is not an instance of OutputFormatterInterface');
  }

  /**
   * {@inheritdoc}
   *
   * Wraps parent method to add typing.
   *
   * @return \Drupal\static_export\Plugin\OutputFormatterInterface
   *   A formatter object instance.
   */
  public function getInstance(array $options): OutputFormatterInterface {
    $instance = parent::getInstance($options);
    if ($instance instanceof OutputFormatterInterface) {
      return $instance;
    }

    throw new PluginException('Plugin is not an instance of OutputFormatterInterface');
  }

}
