<?php

namespace Drupal\static_export\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Entity\EntityInterface;

/**
 * Defines an interface for Resolver plugins.
 */
interface ResolverInterface extends PluginInspectionInterface {

  /**
   * Gets the data for an entity, using an specific resolver.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The Entity to export.
   *
   * @param string|null $variantKey
   *   Variant key, optional.
   *
   * @return array|string
   *   Entity data. An array if resolver supports several output formats, or an
   *   string if not.
   */
  public function resolve(EntityInterface $entity, string $variantKey = NULL);

  /**
   * Get variant keys of an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The Entity to search for variants.
   *
   * @return array
   */
  public function getVariantKeys(EntityInterface $entity): array;

}
