<?php

namespace Drupal\static_export\Semaphore;

use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * A provider of semaphores.
 *
 * At this moment, this only provides data_dir semaphores. In other static_*
 * modules, this functionality is usually provided by a plugin. In
 * static_export we need a SemaphoreProvider since we don't use plugins.
 */
class SemaphoreProvider {

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Semaphore service.
   *
   * @var \Drupal\static_export\Semaphore\SemaphoreService
   */
  protected $semaphoreService;

  /**
   * SemaphoreProvider constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal config factory.
   * @param \Drupal\static_export\Semaphore\SemaphoreService $semaphore_service
   *   Semaphore service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, SemaphoreService $semaphore_service) {
    $this->configFactory = $config_factory;
    $this->semaphoreService = $semaphore_service;
  }

  /**
   * Creates a semaphore for data dir, always the same way.
   *
   * In other static_* modules, this functionality is usually provided by a
   * plugin. In static_export we need a SemaphoreProvider since we don't use
   * plugins.
   *
   * @return Semaphore
   *   Data dir's semaphore.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function createDataDirSemaphore(): Semaphore {
    $dataDir = $this->configFactory->get('static_export.settings')->get('data_dir');
    return $this->semaphoreService->create('data-dir', $dataDir, 'e');
  }

}
