<?php

namespace Drupal\static_export\Semaphore;

use Drupal\static_suite\StaticSuiteException;

/**
 * A service for handling semaphores.
 *
 * Use "ipcs -s" to monitor semaphores.
 */
class SemaphoreService {
  /**
   * An array of Semaphore.
   *
   * @var Semaphore[]
   */
  private $semaphores;

  /**
   * Creates a semaphore.
   *
   * @param string $name
   *   Identifier for this semaphore.
   * @param string $pathname
   *   The pathname of an existing accessible file.
   * @param string $proj
   *   A project identifier of one letter length.
   *
   * @return \Drupal\static_export\Semaphore\Semaphore
   *   A Semaphore.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function create(string $name, string $pathname, string $proj): Semaphore {
    if (!$this->get($name)) {
      $ipcKey = ftok($pathname, $proj);
      if ($ipcKey === -1) {
        throw new StaticSuiteException('Error getting IPC key');
      }
      $this->semaphores[$name] = new Semaphore($ipcKey);
    }
    return $this->get($name);
  }

  /**
   * Get a semaphore.
   *
   * @param string $name
   *   The semaphore identifier.
   *
   * @return Semaphore|null
   *   A semaphore.
   */
  public function get(string $name): ?Semaphore {
    if (!empty($this->semaphores[$name])) {
      return $this->semaphores[$name];
    }
    return NULL;
  }

}
