<?php

namespace Drupal\static_preview_gatsby_instant\PathProcessor;

use Drupal\Core\PathProcessor\InboundPathProcessorInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Request;

/**
 * Processes the inbound path for page-data urls from Gatsby.
 *
 * Since queries with slashes cannot be passed to this path, it replaces
 * slashes with ":", which are later decoded by the controller.
 */
class PageDataPathProcessor implements InboundPathProcessorInterface {

  /**
   * {@inheritdoc}
   */
  public function processInbound($path, Request $request) {
    if (preg_match("#^/page-data/(.*)/page-data.json$#", $path, $matches)) {
      // Add leading slash to $pagePath.
      $pagePath = str_replace("/", ":", "/" . $matches[1]);
      $path = Url::fromRoute('static_preview_gatsby_instant.page-data-resolver', ['pagePath' => $pagePath])->toString();
      return $path;
    }
    return $path;
  }

}
