<?php

namespace Drupal\static_preview_gatsby_instant\PathProcessor;

use Drupal\Core\PathProcessor\InboundPathProcessorInterface;
use Drupal\Core\Url;
use Drupal\static_preview_gatsby_instant\PathResolver\CustomUrlPathResolverInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Processes the inbound path for custom urls defined in configuration.
 *
 * It checks "custom_urls" value to determine if current path is a custom URL
 * that should be handled by a custom controller.
 */
class PageHtmlCustomUrlProcessor implements InboundPathProcessorInterface {

  /**
   * Custom URL path resolver.
   *
   * @var \Drupal\static_preview_gatsby_instant\PathResolver\CustomUrlPathResolverInterface
   */
  protected $customUrlPathResolver;

  /**
   * Constructor.
   *
   * @param \Drupal\static_preview_gatsby_instant\PathResolver\CustomUrlPathResolverInterface $custom_url_path_resolver
   *   Custom URL path resolver.
   */
  public function __construct(CustomUrlPathResolverInterface $custom_url_path_resolver) {
    $this->customUrlPathResolver = $custom_url_path_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public function processInbound($path, Request $request) {
    if ($this->customUrlPathResolver->isCustomUrl($path)) {
      // $path starts with a leading slash.
      $pagePath = str_replace("/", ":", $path);
      $path = Url::fromRoute('static_preview_gatsby_instant.page-html-resolver', ['pagePath' => $pagePath])->toString();
      return $path;
    }
    return $path;
  }

}
