<?php

namespace Drupal\static_preview_gatsby_instant\PathResolver;

use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * {@inheritdoc}
 */
class CustomUrlPathResolver implements CustomUrlPathResolverInterface {

  /**
   * Drupal config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Internal cache for better performance.
   *
   * @var array
   */
  protected $cache;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public function getExportedFilePathRelative(string $pagePath) {
    if (!empty($this->cache[$pagePath])) {
      return $this->cache[$pagePath];
    }

    $customUrls = $this->configFactory->get('static_preview_gatsby_instant.settings')->get('custom_urls');
    if (is_array($customUrls) && count($customUrls) > 0) {
      foreach ($customUrls as $customUrlDefinition) {
        list($pattern, $replacement) = explode(" --- ", $customUrlDefinition);
        if ($pattern && $replacement) {
          if (preg_match("/$pattern/", $pagePath)) {
            $result = preg_replace("/$pattern/", $replacement, $pagePath);
            // Ensure no leading slash is returned.
            $result = preg_replace("/^\//", "", $result);
            $this->cache[$pagePath] = $result;
            return $result;
          }
        }
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getExportedFilePathAbsolute(string $pagePath) {
    $dataDir = $this->configFactory->get('static_export.settings')->get('data_dir');
    $relativeFilePath = $this->getExportedFilePathRelative($pagePath);
    if ($relativeFilePath) {
      $absoluteFilePath = $dataDir . "/" . $relativeFilePath;
      return preg_replace("/\/{2,}/", "/", $absoluteFilePath);
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isCustomUrl(string $pagePath) {
    return $this->getExportedFilePathRelative($pagePath) ? TRUE : FALSE;
  }

}
