<?php

namespace Drupal\static_preview_gatsby_instant\PathResolver;

/**
 * An interface to resolve paths for custom URLs.
 */
interface CustomUrlPathResolverInterface {

  /**
   * Get a custom exported relative file path, based on "custom_urls" config.
   *
   * @param string $pagePath
   *   Page path with leading slash.
   *
   * @return string
   *   Exported relative file path
   */
  public function getExportedFilePathRelative(string $pagePath);

  /**
   * Get a custom exported absolute file path, based on "custom_urls" config.
   *
   * @param string $pagePath
   *   Page path with leading slash.
   *
   * @return string
   *   Exported absolute file path
   */
  public function getExportedFilePathAbsolute(string $pagePath);

  /**
   * Checks whether a given page path (without leading slash) is a custom URL.
   *
   * @param string $pagePath
   *   Page path with leading slash.
   *
   * @return bool
   *   True if $pagePath is a custom URL
   */
  public function isCustomUrl(string $pagePath);

}
