<?php

namespace Drupal\static_preview_subset_build\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\node\Controller\NodeViewController as BaseNodeViewController;
use Drupal\Core\Entity\EntityInterface;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Defines a controller to render a single node.
 */
class NodeViewController extends BaseNodeViewController {

  /**
   * Drupal config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user. For backwards compatibility this is optional, however
   *   this will be removed before Drupal 9.0.0.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    EntityManagerInterface $entity_manager,
    RendererInterface $renderer,
    AccountInterface $current_user
  ) {
    parent::__construct($entity_manager, $renderer, $current_user);
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity.manager'),
      $container->get('renderer'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function view(EntityInterface $node, $view_mode = 'full', $langcode = NULL) {

    // Disable access to unpublished content for authenticated users.
    if ($node instanceof Node) {
      $builtContentTypes = $this->configFactory->get('static_build.settings')->get('built_content_types');
      $status = \Drupal::requestStack()->getCurrentRequest()->attributes->get('exception');
      // We process 404 errors, since it can happen that a node has jus been
      // published for the first time, a build is not yet finished. We must
      // distinguish those 404 hits from the ones that are just 404.
      // There are some situations where $status is null, but this is OK.
      if (
        in_array($node->bundle(), $builtContentTypes) &&
        $this->currentUser->isAuthenticated() &&
        (!$status || in_array($status->getStatusCode(), [200, 404]))
      ) {

        // Distinguish between real 404 and a published node with a build
        // process yet not finished.
        // Get default 404 page.
        $config = $this->configFactory->get('system.site');
        $default404 = $config->get('page.404');
        $default404NodeId = NULL;
        if (!empty($default404) && preg_match("/(\d+)/", $default404, $matches)) {
          $default404NodeId = $matches[1];
        }

        // If we don't get any data from config service, don't try to do
        // anything, since we could be hijacking the default 404 page.
        if ($node->id() != $default404NodeId) {
          if ($node->id() && $node->isPublished()) {
            return ['#markup' => "<p>This page is currently being published. Please, wait a moment and try again.</p>"];
            // Throw new NotFoundHttpException();
          }

          // Try to preview it.
          $url = Url::fromRoute('static_preview_subset_build.preview', ['nid' => $node->id()]);
          return new RedirectResponse($url->toString());
        }

      }
    }

    return parent::view($node, $view_mode, $langcode);
  }

}
