<?php

namespace Drupal\static_suite\Entity;

use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityInterface;

/**
 * An interface for a set of useful utils related to entities.
 */
interface EntityUtilsInterface {

  /**
   * Load an entity.
   *
   * @param string $entityTypeId
   *   The entity type id.
   * @param string $entityId
   *   The entity id.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   Loaded entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   */
  public function loadEntity(string $entityTypeId, string $entityId): ?EntityInterface;

  /**
   * Gets entity ids.
   *
   * @param string $entityTypeId
   *   Entity type id.
   * @param array $options
   *   An associative array with query options:
   *   - bundle: int, entity bundle id
   *   - status: int, entity status, 0 (unpublished) or 1 (published)
   *   - conditions: array of associative arrays
   *     - field: string, name of a field
   *     - value: mixed, the value for field
   *     - operator: same values as Drupal\Core\Entity\Query\QueryInterface::condition()
   *   - sort: associative array
   *     - field: string, name of a field
   *     - direction: string, ASC or DESC
   *   - range: associative array
   *     - start: int, range start
   *     - length: int, range length.
   *
   * @return array
   *   Entity ids
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   */
  public function getEntityIds(string $entityTypeId, $options = []): array;

  /**
   * Gets Entity bundle.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The Entity to get its bundle from.
   *
   * @return string|null
   *   Entity bundle string
   */
  public function getEntityBundleId(EntityInterface $entity): ?string;

  /**
   * Check if entity status has changed.
   *
   * @param \Drupal\Core\Entity\EditorialContentEntityBase $entity
   *   The Entity to be checked.
   *
   * @return bool
   *   True if it has changed
   */
  public function hasStatusChanged(EditorialContentEntityBase $entity): bool;

}
